Applet

Struct Applet 

pub struct Applet {
    pub compiled: Option<MachineCode<f64>>,
    pub compiled_simd: Option<MachineCode<f64>>,
    pub use_simd: bool,
    pub use_threads: bool,
    pub count_states: usize,
    pub count_params: usize,
    pub count_obs: usize,
    pub count_diffs: usize,
}

Fields

compiled: Option<MachineCode<f64>>compiled_simd: Option<MachineCode<f64>>use_simd: booluse_threads: boolcount_states: usizecount_params: usizecount_obs: usizecount_diffs: usize

Implementations

Source

impl Applet

pub fn new(app: Application) -> Result<Applet>

pub fn evaluate<T>(&mut self, args: &[T], outs: &mut [T])
where T: Element,

Generic evaluate function for compiled Symbolica expressions

pub fn evaluate_single<T>(&mut self, args: &[T]) -> T
where T: Element + Copy,

Generic evaluate_single function for compiled Symbolica expressions

pub fn evaluate_matrix<T>(&mut self, args: &[T], outs: &mut [T], n: usize)
where T: Element,

Generic evaluate function for compiled Symbolica expressions The main entry point to compute matrices. The actual dispatched method depends on the configuration and the type of the arguments.