#[unsafe(no_mangle)]pub unsafe extern "C" fn execute_vectorized(
q: *mut CompilerResult,
buf: *mut f64,
n: usize,
) -> boolExpand description
Executes the compiled function n times (vectorized).
The calling function provides buf, which is a k x n matrix of doubles,
where k is equal to the maximum(count_states, count_obs). The calling
funciton fills the first count_states rows of buf. The result is returned
in the first count_obs rows of buf.
Safety
it is the responsibility of the calling function to ensure
that q points to a valid CompilerResult.
In addition, buf should points to a valid matrix of correct size.